function [ODEs] = SynPhoHy(t,sp)
%FullModel: Function to be executed in FullModelScript.

%% Variable Guide %%
%P680=sp(1) P680a=sp(2) P680p=sp(3) QAr=sp(4) QAo=sp(5) PQ=sp(6) PQH2=sp(7)
%ISP_o=sp(8) ISP_r=sp(9) hemebp_o=sp(10) hemebp_r=sp(11) cytf_o=sp(12)
%cytf_r=sp(13) PC_o=sp(14) PC_r=sp(15) P700=sp(16) P700p=sp(17)
%P700a=sp(18) hemebn_o=sp(19) hemebn_r=sp(20) A0_o=sp(21) A0_r=sp(22)
%Fd_o=sp(23) Fd_r=sp(24) NADP=sp(25) NADPH=sp(26) ADP=sp(27) Pi=sp(28)
%ATP=sp(29) H_c=sp(30) H_l=sp(31) G3P=sp(32) NO3=sp(33) NO2=sp(34)
%NH3=sp(35) Ca2+_l=sp(36) Ca2+_c=sp(37) Cl-_l=sp(38) Cl-_c=sp(39)
%K+_l=sp(40) K+_c=sp(41) Mg2+_l=sp(42) Mg2+_c=sp(43) O2=sp(44) H2=sp(45)

% Ks Values Guide for Michaelis-Menten %
    % Very High Affinity = 0.01 
    % High Affinity = 0.1
    % Medium Affinity = 1
    % Low Affinity = 10
    % Very Low Affinity = 100
%% Constants %%
F=9.649*10^4;
V_c=0.25;
V_l=0.75;

% Em %
Em_P680a_P680p= -705;
Em_QAo_QAr= -30;
Em_PQ_PQH2= 82;
Em_ISPo_ISPr= 290;
Em_cytfo_cytfr= 350;
Em_PCo_PCr= 370;
Em_P700p_P700= 440;
Em_P700a_P700p= -1300;
Em_A0o_A0r= -680;
Em_Fdo_Fdr= -440;
Em_NADP_NADPH= -320;
%% pH %%
pH_c= (-log(sp(30)));
pH_l= (-log(sp(31)));
gpH= pH_c-pH_l;
%% Electrochemical Potential %%
NetCharge_l= V_l*F*(2*sp(36) - sp(38) + sp(40) + 2*sp(42));
NetCharge_c= V_c*F*(2*sp(37) - sp(39) + sp(41) + 2*sp(43));
C= 1;
gEP= ((NetCharge_l - NetCharge_c)/C);
%% P680 %%
  
% 1: 2H20 + 4 P680p -> 4 P680 + 4H(l) + O2 
     vmax_ws= 10;
     kP680p_r1= 0.1;
     
  vWS= MM(vmax_ws, sp(3), kP680p_r1); 
            
% 2: 4 P680 + 4hv -> 4 P680a  
     vmax_vP680_P680a=10;
     kP680_P680a= 0.9;
     
  vP680_P680a= MM(vmax_vP680_P680a,sp(1),kP680_P680a) ;

% 3: P680a -> P680
      kP680a_P680= 0.1;

  vP680a_P680= kP680a_P680*sp(2);
  
% 4: 4P680a + 4QA(o) -> 4 P680p + 4QA(r)
      vmax_vP680a_QA= 10;
      kP680a_r4= 0.1;
      kQAo_r4=0.1;
      kP680p_r4=1;
      kQAr_r4=1;
      
  vP680a_QA= MMRE(vmax_vP680a_QA,sp(2),kP680a_r4,sp(5),kQAo_r4,sp(3),kP680p_r4,sp(4),kQAr_r4,1,Em_P680a_P680p,Em_QAo_QAr);
  
% 5: 2QA(r) + 2H+(c) + PQ -> 2QA(o) + PQH2
      vmax_v2QAr_PQH2=1;
      kQAr_r5=0.1;
      kPQ_r5=0.1;
      kQAo_r5=10;
      kPQH2_r5=10;

  v2QAr_PQH2= MMRE(vmax_v2QAr_PQH2,sp(4),kQAr_r5,sp(6),kPQ_r5,sp(5),kQAo_r5,sp(7),kPQH2_r5,2,Em_QAo_QAr,Em_PQ_PQH2);
%% Cytb6f %%
   
% 6: PQH2 + ISP(o) -> SPQ + ISP(r) + 2H+(l) 
      vmax_vPQH2_ISP=10;
      kPQH2_r6=0.1;
      kISPo_r6=0.1;
      kPQ_r6=10;
      kISPr_r6=10;
  
  vPQH2_ISP= MMRE(vmax_vPQH2_ISP,sp(7),kPQH2_r6,sp(8),kISPo_r6,sp(6),kPQ_r6,sp(9),kISPr_r6,1,Em_PQ_PQH2,Em_ISPo_ISPr); 
  
% 7: SPQ + hemebp(o) -> PQ + hemebp(r) 
      
  vSPQ_hemebp= vPQH2_ISP;  
     
      % _High Potential Transport Chain_ %
    
      % 8: ISP(r) + cytf(o) -> ISP(o) + cytf(r)
            vmax_vISP_cytf=1;
            kISPr_r8=0.1;
            kcytfo_r8=0.1;
            kISPo_r8=10;
            kcytfr_r8=10;
      
        vISP_cytf= MMRE(vmax_vISP_cytf,sp(9),kISPr_r8,sp(12), kcytfo_r8,sp(8),kISPo_r8,sp(13),kcytfr_r8,1,Em_ISPo_ISPr,Em_cytfo_cytfr);
          
      % 9: cytf(r) + PC(o) -> cytf(o) + PC(r) 
            vmax_vcytf_PC=1;
            kcytfr_r9=0.1;
            kPCo_r9=0.1;
            kcytfo_r9=10;
            kPCr_r9=10;
           
        vcytf_PC= MMRE(vmax_vcytf_PC,sp(13),kcytfr_r9,sp(14), kPCo_r9,sp(12),kcytfo_r9,sp(15),kPCr_r9,1,Em_cytfo_cytfr,Em_PCo_PCr);
          
      % 10: PC(r) + P700p -> PC(o) + P700
            vmax_vPC_P700=1;
            kPCr_r10=0.1;
            kP700p_r10=0.1;
            kPCo_r10=10;
            kP700_r10=10;
             
        vPC_P700= MMRE(vmax_vPC_P700, sp(15),kPCr_r10,sp(17),kP700p_r10,sp(14),kPCo_r10,sp(16),kP700_r10,1,Em_PCo_PCr,Em_P700p_P700);
          
      % _Low Potential Transport Chain_ %
    
      % 11: hemebp(r) + hemebn(o) -> hemebp(o) + hemebn(r) 
            vmax_vhemebp_hemebn=1;
            khemebpr_r11=0.1;
            khemebno_r11=0.1;
      
        vhemebp_hemebn= MME(vmax_vhemebp_hemebn,sp(11),khemebpr_r11,sp(19),khemebno_r11);
                    
      % 12: 2hemebn(r) + PQ + 2H+(l) -> hemebn(o) + PQH2 
            vmax_vhemebn_PQH2=1;
            khemebnr_r12=0.1;
            kPQ_r12=0.1;
      
        vhemebn_PQH2= MME(vmax_vhemebn_PQH2,sp(20),khemebnr_r12,sp(6),kPQ_r12);
%% P700 %%
    
% 13: P700 + hv -> P700(a) 
      vmax_vP700_P700a=10;
      kP700_P700a=0.1;

  vP700_P700a= MM(vmax_vP700_P700a,sp(16),kP700_P700a);  
% 14: P700a -> P700
      k_vP700a_P700=0.01;

  vP700a_P700= k_vP700a_P700*sp(18);
  
% 15: P700(a) + A0(o) -> P700p + A0(r) 
      vmax_vP700a_A0r=0.1;
      kP700a_r15=0.1; 
      kA0o_r15=0.1;
      kP700p_r15=10;
      kA0r_r15=10;
      
  vP700a_A0r= MMRE(vmax_vP700a_A0r, sp(18),kP700a_r15,sp(21), kA0o_r15,sp(17),kP700p_r15,sp(22),kA0r_r15,1,Em_P700a_P700p,Em_A0o_A0r);
%% Fd & NADPH %%
 
% 16: A0(r) + Fd(o) -> A0(o) + Fd(r) 
      vmax_vA0r_Fdr=1;
      kA0r_r16=0.1;
      kFdo_r16=0.1;
      kA0o_r16=10;
      kFdr_r16=10;

  vA0r_Fdr= MMRE(vmax_vA0r_Fdr,sp(22),kA0r_r16,sp(23),kFdo_r16,sp(21),kA0o_r16,sp(24),kFdr_r16,1,Em_A0o_A0r,Em_Fdo_Fdr);

% 17: 2Fd(r) + NADP+ + H+(c) -> 2Fd(o) + NADPH 
      vmax_vFdr_NADPH=1;
      kFdr_r17=0.1;
      kNADP_r17=0.1;
      kFdo_r17=10;
      kNADPH_r17=10;

  vFdr_NADPH= MMRE(vmax_vFdr_NADPH,sp(24),kFdr_r17,sp(25),kNADP_r17,sp(23),kFdo_r17,sp(26),kNADPH_r17,2,Em_Fdo_Fdr,Em_NADP_NADPH);
%% ATP %%
        
% 18: ADP + Pi + 4,67H+ (l) -> ATP + 4.67H+(c)
      vmax_vADP_ATP=0.1;
      kADP_r18=0.1;
      kPi_r18=0.1;
      kATPr_18=0.1;
      
  vADP_ATP=ATPModule(vmax_vADP_ATP, sp(27), kADP_r18, sp(28), kPi_r18, sp(29), kATPr_18, sp(31), sp(30), gEP);
%% Cyclic Transport %% 
    
      % _Short Cycle_ %
    
      % 19: 2Fd(r)+ PQ + 2H+(s)-> 2Fd(o) + PQH2 
            vmax_vFdr_PQH2=1;
            kFdr_r19=10;
            kPQ_r19=10;
            kFdo_r19=100;
            kPQH2_r19=100;
      
        vFdr_PQH2= MMRE(vmax_vFdr_PQH2,sp(24),kFdr_r19,sp(6),kPQ_r19, sp(23),kFdo_r19,sp(7),kPQH2_r19,2,Em_Fdo_Fdr,Em_PQ_PQH2); 
 
 
      % _Long cycle_ %
    
      % 20: NADPH + PQ -> NADP + PQH2
            vmax_vNADPH_PQH2=1;
            kNADPH_r20=10;
            kPQ_r20=10;
            kNADP_r20=100;
            kPQH2_r20=100;
   
        vNADPH_PQH2= MMRE(vmax_vNADPH_PQH2,sp(26),kNADPH_r20,sp(6),kPQ_r20,sp(25),kNADP_r20,sp(7),kPQH2_r20,2,Em_NADP_NADPH, Em_PQ_PQH2); 
%% Carbon output %%  
 
% 21: 3CO2 + 6 NADPH + 5H20 + 9ATP -> G3P + 2 H_s + 6 NADP + 9 ADP + 8 Pi
      vmax_vCO2_G3P=1;
      kNADPH_r21=0.1;
      kATP_r21=0.1;

  vCO2_G3P= MME(vmax_vCO2_G3P,sp(26),kNADPH_r21,sp(29),kATP_r21); 
 
% 22: G3P -> G3P_ext 
      k_vG3P_G3Pext=1;
      
  vG3P_G3Pext= k_vG3P_G3Pext*sp(32);
%% Nitrogen output %% 
 
% 23: NO3_ext + ATP + H2O -> NO3 + ADP + Pi (90%)
      k_vNO3ext_NO3=0.1;

  vNO3ext_NO3= k_vNO3ext_NO3*sp(29);
   
% 24: N02_ext + ATP + H2O -> NO2 + ADP + PI
      k_vNO2ext_NO2=0.01;  

  vNO2ext_NO2= k_vNO2ext_NO2*sp(29); 
   
% 25: NO3 + 2Fd_r + 2 H_s -> NO2  + 2Fd_o + H2O (10-15%)
      vmax_vNO3_NO2=1;
      kNO3_r25=0.01;
      kFdr_r25=0.1;

  vNO3_NO2= MME(vmax_vNO3_NO2,sp(33),kNO3_r25,sp(24),kFdr_r25);
 
% 26: NO2 + 6 Fd_r + 7 H_s -> NH3 + 6Fd_o + 2H20
      vmax_vNO2_NH3=1;
      kNO2_r26=0.01;
      kFdr_r26=0.1;

  vNO2_NH3= MME(vmax_vNO2_NH3,sp(34),kNO2_r26,sp(24),kFdr_r26);

 % 27: NH3 -> NH3_ext
       k_vNH3_NH3ext=1;
 
   vNH3_NH3ext=k_vNH3_NH3ext*sp(35);
%% Oxygen output %%
 
% 28: 4Fd_r + 4H_s + O2 -> 4Fd_o + 2H2O (Flv 1,3)
      vmax_vFlv1_3=1;
      kFdr_r28=10;
      kO2_r28=1;
  
  vFlv1_3= MME(vmax_vFlv1_3,sp(24),kFdr_r28,sp(44),kO2_r28) ;  
 
% 29: 4PC_r + O2 + 8H_s ->(Cytc)-> 4PC_o + 2H2O + 4H_l
      vmax_vPC_H2O=1;
      kPCr_r29=10;
      kO2_r29=1;
      
  vPC_H2O= MME(vmax_vPC_H2O,sp(15),kPCr_r29,sp(44),kO2_r29);
    
% 30: 2PQH2 + O2 -> (cyt_bd) -> 2PQ + 2H2O 
      vmax_vPQH2_H2O=1;
      kPQH2_r30=10;
      kO2_r30=1;

  vPQH2_H2O= MME(vmax_vPQH2_H2O,sp(7),kPQH2_r30,sp(44),kO2_r30);
 
% 31: O2 -> O2_ext
      k_vO2_O2ext=0.01;

  vO2_O2ext= k_vO2_O2ext*sp(45);
%% Ion fluxes %%
 
% 32: Ca2+(l) -> Ca2+(c)
      P_Ca=1;
  
  vCa2flux= Nernst_Planck(sp(45),sp(46),2,P_Ca,gEP);

% 33: Cl-(l) -> Cl-(c)
      P_Cl=1;
  
  vClflux= Nernst_Planck(sp(47),sp(48),-1,P_Cl,gEP);

% 34: K+(l) -> K+(c)
      P_K=1;
  
  vKflux= Nernst_Planck(sp(49),sp(50),1,P_K,gEP);
  
% 35: Mg2+(l) -> Mg2+(c)
      P_Mg=1;
  
  vMg2flux= Nernst_Planck(sp(51),sp(52),2,P_Mg,gEP);
%% Hydrogen %%

% 36: NADPH + 2 H(c) -> NADP+ + H2
      vmax_vNADPH_H2=100;
      kNADPH_r36=1000;
      kO2i_r36=0.001;
  vNADPH_H2=MMI(vmax_vNADPH_H2,sp(26),kNADPH_r36,sp(44),kO2i_r36); 
  
% 37: 2 Fd(r) + 2H(c) -> 2 Fd(o) + H2
      vmax_vFd_H2=100;
      kFd_r37=10;
      kO2i_r37=0.001;  
      
  vFd_H2=MMI(vmax_vFd_H2,sp(24),kFd_r37,sp(44),kO2i_r37);
%% ODEs %%

% dP680
  dP680 = 4*vWS - 4*vP680_P680a + vP680a_P680;
    
% dP680a
  dP680a = 4*vP680_P680a - 4*vP680a_QA - vP680a_P680;
    
% dP680p
  dP680p = -4*vWS + 4*vP680a_QA;
   
% dQA(o)
  dQA_o = -4*vP680a_QA + 2*v2QAr_PQH2;
      
% dQA(r)
  dQA_r = 4*vP680a_QA - 2*v2QAr_PQH2;

% dPQ
  dPQ = -v2QAr_PQH2 + vPQH2_ISP  + 2*vPQH2_H2O - vFdr_PQH2 - vNADPH_PQH2 - vhemebn_PQH2;
    
% dPQH2
  dPQH2 = v2QAr_PQH2 - vPQH2_ISP + vhemebn_PQH2 - 2*vPQH2_H2O + vFdr_PQH2 + vNADPH_PQH2;
      
% dISP(o)
  dISP_o= -vPQH2_ISP + vISP_cytf;
      
% dISP(r)
  dISP_r = vPQH2_ISP - vISP_cytf;
      
% dcytf(o)
  dcytf_o = -vISP_cytf + vcytf_PC;
      
% dcytf(r)
  dcytf_r= vISP_cytf - vcytf_PC;
      
% dPC(o)
  dPC_o= - vcytf_PC + vPC_P700 + 4*vPC_H2O;
      
% dPC(r)
  dPC_r= vcytf_PC - vPC_P700 - 4*vPC_H2O;

% dP700
  dP700 = vPC_P700 - vP700_P700a + vP700a_P700; 
    
% dP700p
  dP700_p = -vPC_P700 + vP700a_A0r;
  
% dP700a
  dP700_a= vP700_P700a - vP700a_A0r - vP700a_P700;
      
% dhemebp(o)
  dhemebp_o = -vSPQ_hemebp + vhemebp_hemebn ;
      
% dhemebp(r)
  dhemebp_r= vSPQ_hemebp - vhemebp_hemebn  ;
      
% dhemebn(o)
  dhemebn_o= -vhemebp_hemebn + 2*vhemebn_PQH2;
      
% dhemebn(r)
  dhemebn_r= vhemebp_hemebn - 2*vhemebn_PQH2;
  
% dA0(o)  
  dA0_o= -vP700a_A0r + vA0r_Fdr;
  
% dA0(r)
  dA0_r= vP700a_A0r - vA0r_Fdr;
  
% dFd(o)
  dFd_o= -vA0r_Fdr + 2*vFdr_NADPH + 2*vFdr_PQH2 + 2*vNO3_NO2 + 6*vNO2_NH3 + 4*vFlv1_3 + 2*vFd_H2;
  
% dFd(r)
  dFd_r= vA0r_Fdr - 2*vFdr_NADPH - 2*vFdr_PQH2 - 2*vNO3_NO2 - 6*vNO2_NH3 - 4*vFlv1_3 - 2*vFd_H2; 
  
% dNADP+ 
  dNADP = -vFdr_NADPH + 6*vCO2_G3P + vNADPH_PQH2 + vNADPH_H2;

% dNADPH 
  dNADPH = vFdr_NADPH - 6*vCO2_G3P - vNADPH_PQH2 - vNADPH_H2;
  
% dADP  
  dADP= -vADP_ATP + 9*vCO2_G3P + vNO3ext_NO3 + vNO2ext_NO2;

% Pi
  dPi= -vADP_ATP + 9*vCO2_G3P + vNO3ext_NO3 + vNO2ext_NO2;
  
% dATP
  dATP= vADP_ATP - 9*vCO2_G3P - vNO3ext_NO3 - vNO2ext_NO2;

% dH+(l)
  dH_l= (V_c/V_l)*(4*vWS + 2*vPQH2_ISP - 4.67*vADP_ATP + 4*vPC_H2O );
  
% dH+(c)
  dH_s= (- 2*v2QAr_PQH2 - 2*vhemebn_PQH2 + 4.67*vADP_ATP - 2*vNO3_NO2 - 7*vNO2_NH3 - 4*vFlv1_3 - 8*vPC_H2O - 2*vNADPH_PQH2);
  
% dG3P
  dG3P= vCO2_G3P - vG3P_G3Pext;
  
% dNO3
  dNO3= vNO3ext_NO3 - vNO3_NO2;  

% dNO2
  dNO2= vNO2ext_NO2 - vNO2_NH3;

% dNH3
  dNH3= vNO2_NH3 - vNH3_NH3ext;

% dCa2+_l
  dCa2_l= -(vCa2flux)*(V_c/V_l);

% dCa2+_c
  dCa2_c= vCa2flux;

% dCl-_l
  dCl_l= -vClflux*(V_c/V_l);
  
% dCl-_c
  dCl_c= vClflux;
  
% dK+_l
  dK_l= -vKflux*(V_c/V_l);

% dK+_c
  dK_c= vKflux;
  
% dMg2+_l
  dMg2_l= -vMg2flux*(V_c/V_l);

% dMg2+_c
  dMg2_c= vMg2flux;

% dO2
  dO2= vWS - vFlv1_3 - vPC_H2O - vPQH2_H2O - vO2_O2ext; 

% dH2
  dH2= vNADPH_H2 + vFd_H2;
  
% dpH_s 
  dpH_c= pH_c;

% dpH  
  dpH_l= pH_l;

% dgpH  
  dgpH= gpH;
%% Velocities %%

  % vWS
  v1= vWS;
  
  % vP680_P680a
  v2= vP680_P680a;
  
  % vP680a_P680
  v3= vP680a_P680;
  
  % v2QAr_PQH2
  v4= vP680a_QA;
  
  % vPQH2_ISP
  v5= v2QAr_PQH2;
  
  % vSPQ_hemebp
  v6= vPQH2_ISP;
  
  % vISP_cytf
  v7= vSPQ_hemebp;
  
  % vcytf_PC
  v8= vISP_cytf;
  
  % vPC_P700
  v9= vcytf_PC;
  
  % vhemebp_hemebn
  v10= vPC_P700;
  
  % vhemebn_SPQ
  v11= vhemebp_hemebn;
  
  % vhemebn_PQH2
  v12= vhemebn_PQH2;
  
  % vP700r_P700a
  v13= vP700_P700a;
  
  % vP700a_A0r
  v14= vP700a_P700;
  
  % vA0r_P700r
  v15= vP700a_A0r;
  
  % vA0r_Fdr
  v16= vA0r_Fdr;
  
  % vFdr_NADPH
  v17= vFdr_NADPH;
  
  % vADP_ATP
  v18= vADP_ATP;
 
  % vFdr_PQH2
  v19= vFdr_PQH2;
  
  % vNADPH_PQH2
  v20= vNADPH_PQH2;
  
  % vCO2_G3P
  v21= vCO2_G3P;
  
  % vG3P_G3Pext
  v22= vG3P_G3Pext;
  
  % vNO3ext_NO3
  v23= vNO3ext_NO3;
  
  % vNO2ext_NO2
  v24= vNO2ext_NO2;
  
  % vNO3_NO2
  v25= vNO3_NO2;
  
  % vNO2_NH3
  v26= vNO2_NH3;
  
  % vNH3_NH3ext
  v27= vNH3_NH3ext;
  
  % vFlv1_3
  v28= vFlv1_3;
  
  % vPC_H2O
  v29= vPC_H2O;
    
  % vPQH2_H2O
  v30= vPQH2_H2O;
  % vO2_O2ext
  v31= vO2_O2ext;
  
  % vCa2+flux
  v32= vCa2flux;
  
  % vCl-flux
  v33= vClflux;
  
  % vK+flux
  v34= vKflux;
  
  % vMg2+flux
  v35= vMg2flux;
  
  % vNADPH_H2
  v36= vNADPH_H2;
  
  %vFd_H2
  v37= vFd_H2;
%% Balances %% 
e_balance= (4*v1) - (12*v21 + 2*v25 + 6*v26 + 2*v28 + 4*v29 + 4*v30 +2*v36 +2*v37) ;
%% [ODEs] %%

ODEs= [ dP680 dP680a dP680p dQA_r dQA_o dPQ dPQH2 dISP_o dISP_r dhemebp_o dhemebp_r dcytf_o dcytf_r dPC_o dPC_r dP700 dP700_p dP700_a dhemebn_o dhemebn_r dA0_o dA0_r dFd_o dFd_r dNADP dNADPH dADP dPi dATP dH_s dH_l dG3P dNO3 dNO2 dNH3 dCa2_l dCa2_c dCl_l dCl_c dK_l dK_c dMg2_l dMg2_c dO2 dH2 dpH_c dpH_l dgpH v1 v2 v3 v4 v5 v6 v7 v8 v9 v10 v11 v12 v13 v14 v15 v16 v17 v18 v19 v20 v21 v22 v23 v24 v25 v26 v27 v28 v29 v30 v31 v32 v33 v34 v35 v36 v37 e_balance ]';
end

